=== modified file 'bzrlib/tests/test_hashcache.py'
--- bzrlib/tests/test_hashcache.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/test_hashcache.py	2005-12-26 05:28:38 +0000
@@ -61,21 +61,21 @@
         # check we hit without re-reading
         self.assertEquals(hc.get_sha1('foo'),
                           'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        ##self.assertEquals(hc.miss_count, 1)
-        ##self.assertEquals(hc.hit_count, 1)
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 1)
 
         # check again without re-reading
         self.assertEquals(hc.get_sha1('foo'),
                           'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        ##self.assertEquals(hc.miss_count, 1)
-        ##self.assertEquals(hc.hit_count, 2)
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 2)
 
         # write new file and make sure it is seen
         file('foo', 'wb').write('goodbye')
         pause()
         self.assertEquals(hc.get_sha1('foo'),
                           '3c8ec4874488f6090a157b014ce3397ca8e06d4f')
-        ##self.assertEquals(hc.miss_count, 2)
+        self.assertEquals(hc.miss_count, 2)
 
         # quickly write new file of same size and make sure it is seen
         # this may rely on detection of timestamps that are too close
@@ -101,9 +101,9 @@
 
         # should now be safe to cache it if we reread them
         self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        ##self.assertEquals(len(hc._cache), 1)
+        self.assertEquals(len(hc._cache), 1)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-        ##self.assertEquals(len(hc._cache), 2)
+        self.assertEquals(len(hc._cache), 2)
 
         # write out, read back in and check that we don't need to
         # re-read any files
@@ -113,8 +113,8 @@
         hc = HashCache(u'.')
         hc.read()
 
-        ##self.assertEquals(len(hc._cache), 2)
+        self.assertEquals(len(hc._cache), 2)
         self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        ##self.assertEquals(hc.hit_count, 1)
-        ##self.assertEquals(hc.miss_count, 0)
+        self.assertEquals(hc.hit_count, 1)
+        self.assertEquals(hc.miss_count, 0)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2005-12-01 23:52:16 +0000
+++ bzrlib/tests/test_workingtree.py	2006-01-02 22:54:45 +0000
@@ -171,3 +171,17 @@
         self.assertEquals(list(tree.unknowns()),
                           ['hello.txt'])
 
+    def test_hashcache(self):
+        from bzrlib.tests.test_hashcache import pause
+        b = Branch.initialize(u'.')
+        tree = WorkingTree(u'.', b)
+        self.build_tree(['hello.txt',
+                         'hello.txt~'])
+        tree.add('hello.txt')
+        pause()
+        sha = tree.get_file_sha1(tree.path2id('hello.txt'))
+        self.assertEqual(1, tree._hashcache.miss_count)
+        tree2 = WorkingTree(u'.', b)
+        sha2 = tree2.get_file_sha1(tree2.path2id('hello.txt'))
+        self.assertEqual(0, tree2._hashcache.miss_count)
+        self.assertEqual(1, tree2._hashcache.hit_count)

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-12-16 03:43:14 +0000
+++ bzrlib/workingtree.py	2006-01-02 22:54:45 +0000
@@ -197,8 +197,6 @@
         self.branch = branch
         self.basedir = realpath(basedir)
 
-        self._set_inventory(self.read_working_inventory())
-
         # update the whole cache up front and write to disk if anything changed;
         # in the future we might want to do this more selectively
         # two possible ways offer themselves : in self._unlock, write the cache
@@ -213,6 +211,8 @@
             mutter("write hc")
             hc.write()
 
+        self._set_inventory(self.read_working_inventory())
+
     def _set_inventory(self, inv):
         self._inventory = inv
         self.path2id = self._inventory.path2id
@@ -318,6 +318,7 @@
     def get_file_size(self, file_id):
         return os.path.getsize(self.id2abspath(file_id))
 
+    @needs_read_lock
     def get_file_sha1(self, file_id):
         path = self._inventory.id2path(file_id)
         return self._hashcache.get_sha1(path)
@@ -913,6 +914,8 @@
         between multiple working trees, i.e. via shared storage, then we 
         would probably want to lock both the local tree, and the branch.
         """
+        if self._hashcache.needs_write:
+            self._hashcache.write()
         return self.branch.unlock()
 
     @needs_write_lock

